
/*
** FASM - 6502 assembler
** Copyright (c) 2000 by Toshiyasu Morita
** Released under the terms of the GNU Public License
** See the file COPYING for details
*/

/*
** Header files
*/

#include <stdio.h>
#include <stdlib.h>

#include "proto.h"
#include "fasm.h"

/*
** Convert character to uppercase
*/

int Convert_Character(char c)

{

	if ( (c >= 'a') && (c <= 'z') )
		return(c - 'a');

	if ( (c >= 'A') && (c <= 'Z') )
		return(c - 'A');

	if (!c)
		return(26);

	return(-1);


}

/*
** Identify instruction
*/

int Identify_Instruction()

{

	char c, *string = operator;

	int state;

	static UBYTE state_table[105][27] = {


/*           a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x   y   z  00  */

/*   0 */    1,  8, 27, 38, 43,  0,  0,  0, 46, 51,  0, 56,  0, 63, 66, 69,  0, 76, 83, 94,  0,  0,  0,  0,  0,  0,  0,  /*   0 */
/*   1 */    0,  0,  0,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  0,  6,  0,  0,  0,  0,  0,  0,  0,  0,  /*   1 */
/*   2 */    0,  0,  3,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*   2 */
/*   3 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,128,  /*   3 */
/*   4 */    0,  0,  0,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*   4 */
/*   5 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,129,  /*   5 */
/*   6 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  7,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*   6 */
/*   7 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,130,  /*   7 */
/*   8 */    0,  0,  9,  0, 12,  0,  0,  0, 14,  0,  0,  0, 16, 18,  0, 20,  0, 22,  0,  0,  0, 24,  0,  0,  0,  0,  0,  /*   8 */
/*   9 */    0,  0, 10,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 11,  0,  0,  0,  0,  0,  0,  0,  0,  /*   9 */
/*  10 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,131,  /*  10 */

/*  11 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,132,  /*  11 */
/*  12 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 13,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  12 */
/*  13 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,133,  /*  13 */
/*  14 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 15,  0,  0,  0,  0,  0,  0,  0,  /*  14 */
/*  15 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,134,  /*  15 */
/*  16 */    0,  0,  0,  0,  0,  0,  0,  0, 17,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  16 */
/*  17 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,135,  /*  17 */
/*  18 */    0,  0,  0,  0, 19,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  18 */
/*  19 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,136,  /*  19 */

/*  20 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 21,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  20 */
/*  21 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,137,  /*  21 */
/*  22 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 23,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  22 */
/*  23 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,138,  /*  23 */
/*  24 */    0,  0, 25,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 26,  0,  0,  0,  0,  0,  0,  0,  0,  /*  24 */
/*  25 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,139,  /*  25 */
/*  26 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,140,  /*  26 */
/*  27 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 28, 33,  0,  0, 35,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  27 */
/*  28 */    0,  0, 29, 30,  0,  0,  0,  0, 31,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 32,  0,  0,  0,  0,  0,  /*  28 */
/*  29 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,141,  /*  29 */

/*  30 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,142,  /*  30 */
/*  31 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,143,  /*  31 */
/*  32 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,144,  /*  32 */
/*  33 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 34,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  33 */
/*  34 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,145,  /*  34 */
/*  35 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 36, 37,  0,  0,  /*  35 */
/*  36 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,146,  /*  36 */
/*  37 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,147,  /*  37 */
/*  38 */    0,  0,  0,  0, 39,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  38 */
/*  39 */    0,  0, 40,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 41, 42,  0,  0,  /*  39 */

/*           a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x   y   z  00  */

/*  40 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,148,  /*  40 */
/*  41 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,149,  /*  41 */
/*  42 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,150,  /*  42 */
/*  43 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 44,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  43 */
/*  44 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 45,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  44 */
/*  45 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,151,  /*  45 */
/*  46 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 47,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  46 */
/*  47 */    0,  0, 48,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 49, 50,  0,  0,  /*  47 */
/*  48 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,152,  /*  48 */
/*  49 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,153,  /*  49 */

/*  50 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,154,  /*  50 */
/*  51 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 52,  0,  0,  0,  0,  0, 54,  0,  0,  0,  0,  0,  0,  0,  0,  /*  51 */
/*  52 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 53,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  52 */
/*  53 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,155,  /*  53 */
/*  54 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 55,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  54 */
/*  55 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,156,  /*  55 */
/*  56 */    0,  0,  0, 57,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 61,  0,  0,  0,  0,  0,  0,  0,  0,  /*  56 */
/*  57 */   58,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 59, 60,  0,  0,  /*  57 */
/*  58 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,157,  /*  58 */
/*  59 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,158,  /*  59 */

/*  60 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,159,  /*  60 */
/*  61 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 62,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  61 */
/*  62 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,160,  /*  62 */
/*  63 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 64,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  63 */
/*  64 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 65,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  64 */
/*  65 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,161,  /*  65 */
/*  66 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 67,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  66 */
/*  67 */   68,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  67 */
/*  68 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,162,  /*  68 */
/*  69 */    0,  0,  0,  0,  0,  0,  0, 70,  0,  0,  0, 73,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  69 */

/*  70 */   71,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 72,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  70 */
/*  71 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,163,  /*  71 */
/*  72 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,164,  /*  72 */
/*  73 */   74,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 75,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  73 */
/*  74 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,165,  /*  74 */
/*  75 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,166,  /*  75 */
/*  76 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 77,  0,  0,  0,  0, 80,  0,  0,  0,  0,  0,  0,  0,  /*  76 */
/*  77 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 78,  0,  0,  0,  0,  0, 79,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  77 */
/*  78 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,167,  /*  78 */
/*  79 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,168,  /*  79 */

/*           a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x   y   z  00  */

/*  80 */    0,  0,  0,  0,  0,  0,  0,  0, 81,  0,  0,  0,  0,  0,  0,  0,  0,  0, 82,  0,  0,  0,  0,  0,  0,  0,  0,  /*  80 */
/*  81 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,169,  /*  81 */
/*  82 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,170,  /*  82 */
/*  83 */    0, 84,  0,  0, 86,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 90,  0,  0,  0,  0,  0,  0,  0,  /*  83 */
/*  84 */    0,  0, 85,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  84 */
/*  85 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,171,  /*  85 */
/*  86 */    0,  0, 87, 88,  0,  0,  0,  0, 89,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /*  86 */
/*  87 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,172,  /*  87 */
/*  88 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,173,  /*  88 */
/*  89 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,174,  /*  89 */

/*  90 */   91,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 92, 93,  0,  0,  /*  90 */
/*  91 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,175,  /*  91 */
/*  92 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,176,  /*  92 */
/*  93 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,177,  /*  93 */
/*  94 */   95,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 98,  0,  0,  0,  0,100,103,  0,  0,  /*  94 */
/*  95 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 96, 97,  0,  0,  /*  95 */
/*  96 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,178,  /*  96 */
/*  97 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,179,  /*  97 */
/*  98 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 99,  0,  0,  0,  /*  98 */
/*  99 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,180,  /*  99 */

/* 100 */  101,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,102,  0,  0,  0,  0,  0,  0,  0,  0,  /* 100 */
/* 101 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,181,  /* 101 */
/* 102 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,182,  /* 102 */
/* 103 */  104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  /* 103 */
/* 104 */    0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,183,  /* 104 */

};

	state = 0;

	while( (state < 128)) {

		if ( (c = Convert_Character(*string++)) == -1)
			return(-1);

		if ( !(state = state_table[state][c]) )
			return(-1);

		}

	return(state & 0x07f);

}



/*
** Encode instruction to hex value
*/

int Encode_Instruction(int instruction, int addr_mode)

{

	int code;

    static int value[56][13] = {

/*            impl  accum immed PCREL ABS   ZP    ABS,X ZP,X  ABS,Y ZP,Y  (ABS) (ZP,X)(ZP),Y

/* ADC  0 */    -1,   -1,0x069,   -1,0x06d,0x065,0x07d,0x075,0x079,   -1,   -1,0x061,0x071,
/* AND  1 */    -1,   -1,0x029,   -1,0x02d,0x025,0x03d,0x035,0x039,   -1,   -1,0x021,0x031,
/* ASL  2 */    -1,0x00a,   -1,   -1,0x00e,0x006,0x01e,0x016,   -1,   -1,   -1,   -1,   -1,
/* BCC  3 */    -1,   -1,   -1,0x090,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BCS  4 */    -1,   -1,   -1,0x0b0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BEQ  5 */    -1,   -1,   -1,0x0f0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BIT  6 */    -1,   -1,   -1,   -1,0x02c,0x024,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BMI  7 */    -1,   -1,   -1,0x030,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BNE  8 */    -1,   -1,   -1,0x0d0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BPL  9 */    -1,   -1,   -1,0x010,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BRK 10 */ 0x000,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BVC 11 */    -1,   -1,   -1,0x050,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* BVS 12 */    -1,   -1,   -1,0x070,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* CLC 13 */ 0x018,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* CLD 14 */ 0x0d8,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* CLI 15 */ 0x058,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* CLV 16 */ 0x0b8,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* CMP 17 */    -1,   -1,0x0c9,   -1,0x0cd,0x0c5,0x0dd,0x0d5,0x0d9,   -1,   -1,0x0c1,0x0d1,
/* CPX 18 */    -1,   -1,0x0e0,   -1,0x0ec,0x0e4,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* CPY 19 */    -1,   -1,0x0c0,   -1,0x0cc,0x0c4,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* DEC 20 */    -1,   -1,   -1,   -1,0x0ce,0x0c6,0x0de,0x0d6,   -1,   -1,   -1,   -1,   -1,
/* DEX 21 */ 0x0ca,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* DEY 22 */ 0x088,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* EOR 23 */    -1,   -1,0x049,   -1,0x04d,0x045,0x05d,0x055,0x059,   -1,   -1,0x041,0x051,
/* INC 24 */    -1,   -1,   -1,   -1,0x0ee,0x0e6,0x0fe,0x0f6,   -1,   -1,   -1,   -1,   -1,
/* INX 25 */ 0x0e8,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* INY 26 */ 0x0c8,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* JMP 27 */    -1,   -1,   -1,   -1,0x04c,   -1,   -1,   -1,   -1,   -1,0x06c,   -1,   -1,

/*            impl  accum immed PCREL ABS   ZP    ABS,X ZP,X  ABS,Y ZP,Y  (ABS) (ZP,X)(ZP),Y

/* JSR 28 */    -1,   -1,   -1,   -1,0x020,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* LDA 29 */    -1,   -1,0x0a9,   -1,0x0ad,0x0a5,0x0bd,0x0b5,0x0b9,   -1,   -1,0x0a1,0x0b1,
/* LDX 30 */    -1,   -1,0x0a2,   -1,0x0ae,0x0a6,   -1,   -1,0x0be,0x0b6,   -1,   -1,   -1,
/* LDY 31 */    -1,   -1,0x0a0,   -1,0x0ac,0x0a4,0x0bc,0x0b4,   -1,   -1,   -1,   -1,   -1,
/* LSR 32 */    -1,0x04a,   -1,   -1,0x04e,0x046,0x05e,0x056,   -1,   -1,   -1,   -1,   -1,
/* NOP 33 */ 0x0ea,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* ORA 34 */    -1,   -1,0x009,   -1,0x00d,0x005,0x01d,0x015,0x019,   -1,   -1,0x001,0x011,
/* PHA 35 */ 0x048,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* PHP 36 */ 0x008,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* PLA 37 */ 0x068,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* PLP 38 */ 0x028,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* ROL 39 */    -1,0x02a,   -1,   -1,0x02e,0x026,0x03e,0x036,   -1,   -1,   -1,   -1,   -1,
/* ROR 40 */    -1,0x06a,   -1,   -1,0x06e,0x066,0x07e,0x076,   -1,   -1,   -1,   -1,   -1,
/* RTI 41 */ 0x040,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* RTS 42 */ 0x060,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* SBC 43 */    -1,   -1,0x0e9,   -1,0x0ed,0x0e5,0x0fd,0x0f5,0x0f9,   -1,   -1,0x0e1,0x0f1,
/* SEC 44 */ 0x038,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* SED 45 */ 0x0f8,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* SEI 46 */ 0x078,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* STA 47 */    -1,   -1,   -1,   -1,0x08d,0x085,0x09d,0x095,0x099,   -1,   -1,0x081,0x091,
/* STX 48 */    -1,   -1,   -1,   -1,0x08e,0x086,   -1,   -1,   -1,0x096,   -1,   -1,   -1,
/* STY 49 */    -1,   -1,   -1,   -1,0x08c,0x084,   -1,0x094,   -1,   -1,   -1,   -1,   -1,
/* TAX 50 */ 0x0aa,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* TAY 51 */ 0x0a8,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* TSX 52 */ 0x0ba,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* TXA 53 */ 0x08a,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* TXS 54 */ 0x09a,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
/* TYA 55 */ 0x098,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1

};

	/* Input addressing mode:

		00 = implied
		01 = accumulator
		02 = immediate
		03 = ABS/ZP
		04 = ABS,X/ZP,X
		05 = ABS,Y/ZP,Y
		06 = (ABS)
		07 = (ZP,X)
		08 = (ZP),Y

	   Table addressing mode:

		00 = implied
		01 = accumulator
		02 = immediate
		03 = PC relative
		04 = ABS
		05 = ZP
		06 = ABS,X
		07 = ZP,X
		08 = ABS,Y
		09 = ZP,Y
		10 = (ABS)
		11 = (ZP,X)
		12 = (ZP),Y */

	switch (addr_mode) {


		case 0: 	/* implied     	Easy - no translation */
		case 1: 	/* accumulator                        */

			label2_type = 0;
			return(value[instruction][addr_mode]);


		case 2: 	/* immediate    Easy - no translation */

			label2_type = 1;
			return(value[instruction][2]);


		case 3: 	/* ABS/ZP    	Gets tricky - have to figure out */
					/*              Whether encode instruction as    */
					/*              ABS, ZP, or PC relative          */


					/* Try ZP first */

			if ( ((code = value[instruction][5]) != ERROR) && (!(label2_addr & 0xffffff00)) ) {
				label2_type = 1;
				return(code);
				}


			if ( (code = value[instruction][4]) != ERROR ) {     	/* Next try ABS */
				label2_type = 2;
				return(code);
				}
			else {
				label2_type = 3;
				return(value[instruction][3]);	/* Else return PC rel */
				}


		case 4:   	/* ABS,X/ZP,X   Easy - some translation */

			if ( ((code = value[instruction][7]) != ERROR) && (!(label2_addr & 0xffffff00)) ) {
				label2_type = 1;
				return(code);
				}
			else {
				label2_type = 2;
				return(value[instruction][6]);
				}


		case 5:	/* ABS,Y/ZP,Y	Easy - some translation */

			if ( ((code = value[instruction][9]) != ERROR) && (!(label2_addr & 0xffffff00)) ) {
				label2_type = 1;
				return(code);
				}
			else {
				label2_type = 2;
				return(value[instruction][8]);
				}


		case 6:  	/* (ABS) 		Easy - no translation */

			label2_type = 2;
			return(value[instruction][10]);


		case 7:		/* (ZP,X)		Easy - no translation */

			label2_type = 1;
			return(value[instruction][11]);


		case 8:		/* (ZP),Y		Easy - no translation */

			label2_type = 1;
			return(value[instruction][12]);


		case 9:		/* immediate low */

			label2_type = 4;
			return(value[instruction][2]);


		case 10:    /* immediate high */

			label2_type = 5;
			return(value[instruction][2]);


		case 11: 	/* immediate bank */

			label2_type = 6;
			return(value[instruction][2]);


		default:  	/* This shouldn't happen! */

			printf("Internal assembler error!\n");
			printf("Encode_Instruction: Unknown addressing mode received! (%d)\n", addr_mode);

			label2_type = 1;
			return(ERROR);
		}
}
